/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      MemCopyListMode.c
 * @brief     Example to demonstrate memory copy using list mode.
 * @version:  $Revision: 17899 $
 * @date:     $Date: 2014-06-05 10:05:00 -0400 (Thu, 05 Jun 2014) $
 *
 * @details
 *            This is the primary source file to demonstrate
 *            memory copy using DMA list mode with Memory DMA manager APIs.
 *
 */

/*=============  I N C L U D E S   =============*/

/* List mode Memory Copy example includes */
#include "MemCopy.h"
/* Managed drivers and/or services include */
#include "adi_initialize.h"

/*=============  D A T A  =============*/

/* DMA Stream Handle */
ADI_DMA_STREAM_HANDLE   hMemDmaStream0;
ADI_DMA_STREAM_HANDLE   hMemDmaStream1;
ADI_DMA_STREAM_HANDLE   hMemDmaStream2;
/* Source DMA Handle */
ADI_DMA_CHANNEL_HANDLE  hSrcDmaChannel0;
ADI_DMA_CHANNEL_HANDLE  hSrcDmaChannel1;
ADI_DMA_CHANNEL_HANDLE  hSrcDmaChannel2;
/* Destination DMA Handle */
ADI_DMA_CHANNEL_HANDLE  hDestDmaChannel0;
ADI_DMA_CHANNEL_HANDLE  hDestDmaChannel1;
ADI_DMA_CHANNEL_HANDLE  hDestDmaChannel2;

/* Memory to handle DMA Stream */
static uint8_t MemDmaStreamMem0[ADI_DMA_STREAM_REQ_MEMORY];
static uint8_t MemDmaStreamMem1[ADI_DMA_STREAM_REQ_MEMORY];
static uint8_t MemDmaStreamMem2[ADI_DMA_STREAM_REQ_MEMORY];

/*=============   GLOBAL VARS  =============*/
/* Flag to register Memory DMA copy completion */
volatile bool bMem0CopyInProgress;
volatile bool bMem1CopyInProgress;
volatile bool bMem2CopyInProgress;


/*=============  L O C A L    F U N C T I O N S  =============*/


/* Memory DMA callback */
static void MemDmaCallback(void *AppHandle, uint32_t Event, void *pArg);


/*=============  C O D E  =============*/

/*
 *  Callback from MDMA Manager
 *
 * Parameters
 *  - [in]  pCBParam    Callback parameter supplied by application
 *  - [in]  Event       Callback event
 *  - [in]  pArg        Callback argument
 *
 * Returns  None
 *
 */
static void MemDmaCallback(void *pCBParam, uint32_t Event, void *pArg)
{
    /* CASEOF (Event) */
    switch (Event)
    {
    	/* CASE (Descriptor List Processed) */
    	case (ADI_DMA_EVENT_BUFFER_PROCESSED):
    	    /* Update memory copy status flag */
    	    if(pCBParam == hMemDmaStream0){
    	    	bMem0CopyInProgress = false;
    	    }else if (pCBParam == hMemDmaStream1){
    	    	bMem1CopyInProgress = false;
    	    }else if (pCBParam == hMemDmaStream2){
    	    	bMem2CopyInProgress = false;
    	    }
    		break;
    }
}



#ifdef __HAS_SOFT_CONFIG__
/* Sets the Software controlled switches*/
extern void ConfigSoftSwitches(void);
#endif


/*********************************************************************
*
*   Function:   Configure MDMA 0,1,2
*
*********************************************************************/
void MDMAprep (void)
{

    /* DMA Return code */
    ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;


#ifdef __HAS_SOFT_CONFIG__
    /* Set the Software controlled switches to default values */
    ConfigSoftSwitches();
#endif

    /* IF (Success) */
    if (eResult == ADI_DMA_SUCCESS)
    {
    	ADI_DMA_RESULT      eResult0, eResult1, eResult2;
    	eResult0 = eResult1 = eResult2 = ADI_DMA_SUCCESS;
        /* Open Memory DMA 0 Stream */
        eResult0 = adi_mdma_Open (ADI_DMA_MEMDMA_S0,
                                 &MemDmaStreamMem0[0],
                                 &hMemDmaStream0,
                                 &hSrcDmaChannel0,
                                 &hDestDmaChannel0,
                                 NULL,
                                 NULL);
#if 1
        /* IF (Failure) */
        if (eResult0 != ADI_DMA_SUCCESS)
        {
        	DEBUG_MSG2("Failed to open MDMA 0 stream", eResult0);
        }

        /* Open Memory DMA 0 Stream */
		eResult1 = adi_mdma_Open (ADI_DMA_MEMDMA_S1,
								 &MemDmaStreamMem1[0],
								 &hMemDmaStream1,
								 &hSrcDmaChannel1,
								 &hDestDmaChannel1,
								 MemDmaCallback,
								 hMemDmaStream1);

		/* IF (Failure) */
		if (eResult1 != ADI_DMA_SUCCESS)
		{
			DEBUG_MSG2("Failed to open MDMA 1 stream", eResult1);
		}

		/* Open Memory DMA 0 Stream */
		eResult2 = adi_mdma_Open (ADI_DMA_MEMDMA_S2,
								 &MemDmaStreamMem2[0],
								 &hMemDmaStream2,
								 &hSrcDmaChannel2,
								 &hDestDmaChannel2,
								 MemDmaCallback,
								 hMemDmaStream2);

		/* IF (Failure) */
		if (eResult2 != ADI_DMA_SUCCESS)
		{
			DEBUG_MSG2("Failed to open MDMA 0 stream", eResult2);
		}

#endif
		eResult = eResult2 | eResult1 | eResult0;
    }

    if (eResult == ADI_DMA_SUCCESS)
	{
		/* Set DMA Callback Function. No need to set for source channel since callback is not supported for it */
		eResult = adi_dma_UpdateCallback (hDestDmaChannel0, MemDmaCallback, hMemDmaStream0);

		/* IF (Failure) */
		if (eResult != ADI_DMA_SUCCESS)
		{
			DEBUG_MSG2("Failed to set DMA callback", eResult);
		}
#if 1
		/* Set DMA Callback Function. No need to set for source channel since callback is not supported for it */
		eResult = adi_dma_UpdateCallback (hDestDmaChannel1, MemDmaCallback, hMemDmaStream1);

		/* IF (Failure) */
		if (eResult != ADI_DMA_SUCCESS)
		{
			DEBUG_MSG2("Failed to set DMA callback", eResult);
		}

		/* Set DMA Callback Function. No need to set for source channel since callback is not supported for it */
		eResult = adi_dma_UpdateCallback (hDestDmaChannel2, MemDmaCallback, hMemDmaStream2);

		/* IF (Failure) */
		if (eResult != ADI_DMA_SUCCESS)
		{
			DEBUG_MSG2("Failed to set DMA callback", eResult);
		}
#endif
	}


}

ADI_DMA_RESULT StartOneShotTransmission(ADI_DMA_STREAM_ID id, void * dest, void * src){
	/* DMA Return code */
	ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;
	switch (id){
	case ADI_DMA_MEMDMA_S0:
		eResult = adi_mdma_Copy1D(hMemDmaStream0, dest, src, MEMCOPY_MSIZE, MY_IMG_BLK_SIZE/MEMCOPY_MSIZE_IN_BYTES);
		bMem0CopyInProgress = true;
		break;
	case ADI_DMA_MEMDMA_S1:
		eResult = adi_mdma_Copy1D (hMemDmaStream1, dest, src, MEMCOPY_MSIZE, MY_IMG_BLK_SIZE/MEMCOPY_MSIZE_IN_BYTES);
		bMem1CopyInProgress = true;
		break;
	case ADI_DMA_MEMDMA_S2:
		eResult = adi_mdma_Copy1D (hMemDmaStream2, dest, src, MEMCOPY_MSIZE, MY_IMG_BLK_SIZE/MEMCOPY_MSIZE_IN_BYTES);
		bMem2CopyInProgress = true;
	}

	if (eResult != ADI_DMA_SUCCESS)
	{
		DEBUG_MSG2("Failed to start MDMA", eResult);
	}

	return eResult;

}





/*
** EOF
*/
